#!/usr/bin/env python3
# Time-stamp: "2024-01-28 09:37:16 (ywatanabe)"

from . import dsp
from . import general
from . import general as gen
from . import io, linalg, ml, nn, plt, resource, stats
from .general.debug import *

__copyright__ = "Copyright (C) 2021 Yusuke Watanabe"
__version__ = "0.10.1"
__license__ = "GPL3.0"
__author__ = "ywatanabe1989"
__author_email__ = "ywata1989@gmail.com"
__url__ = "https://github.com/ywatanabe1989/mngs"
