#!/usr/bin/env python3

from . import colors
from ._ax_extend import ax_extend
from ._ax_scientific_notation import ax_scientific_notation
from ._ax_set_n_ticks import ax_set_n_ticks
from ._ax_set_position import ax_set_position
from ._ax_set_size import ax_set_size
from ._configure_mpl import configure_mpl
from ._draw_a_cube import draw_a_cube
from ._get_RGBA_from_colormap import get_RGBA_from_colormap
from .annotated_heatmap import annotated_heatmap
from ._mk_patches import mk_patches
from ._mk_colorbar import mk_colorbar
from ._add_hue import add_hue
from ._ax_fill_between import ax_fill_between
from ._ax_circular_hist import ax_circular_hist
